@@ -25,6 +25,19 @@ TOP = round(297 * 72 / 25.4, 6)
 RIGHT = round(210 * 72 / 25.4, 6)
 
 
+def assert_rect_almost_equal(rect, values):
+    """Test that PDF rect string equals given values.
+
+    We avoid rounding errors by allowing a delta of 1, as both WeasyPrint and
+    pydyf round coordinates in unpredictable ways.
+
+    """
+    if isinstance(rect, bytes):
+        rect = rect.decode('ascii')
+    for a, b in zip(rect.strip(' []').split(), values):
+        assert abs(int(a) - b) <= 1
+
+
 @assert_no_logs
 @pytest.mark.parametrize('zoom', (1, 1.5, 0.5))
 def test_page_size_zoom(zoom):
@@ -399,30 +412,13 @@ def test_relative_links_anchors():
     assert [float(number) for number in rect.split()] == [0, TOP, RIGHT, TOP]
 
 
-@assert_no_logs
-def test_relative_links_different_base():
-    pdf = FakeHTML(
-        string='<a href="/test/lipsum"></a>a',
-        base_url='http://weasyprint.org/foo/bar/').write_pdf()
-    assert b'http://weasyprint.org/test/lipsum' in pdf
-
-
-@assert_no_logs
-def test_relative_links_same_base():
-    pdf = FakeHTML(
-        string='<a id="test" href="/foo/bar/#test"></a>a',
-        base_url='http://weasyprint.org/foo/bar/').write_pdf()
-    assert b'/Dest (test)' in pdf
-
-
 @assert_no_logs
 def test_missing_links():
     with capture_logs() as logs:
         pdf = FakeHTML(string='''
           <style> a { display: block; height: 15pt } </style>
           <a href="#lipsum"></a>
-          <a href="#missing" id="lipsum"></a>
-          <a href=""></a>a
+          <a href="#missing" id="lipsum"></a>a
         ''', base_url=None).write_pdf()
     assert b'/Dest (lipsum)' in pdf
     assert len(logs) == 1
